'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.unwrap = undefined;
exports.default = read;

var _immutable = require('immutable');

var BASE = 10;
var DOT = '.';
var ID = ':';
var ALL = '*';
var AllChildren = '*.';

var splitHistory = {};

var isInString = function isInString(string, toFind) {
  return string.indexOf(toFind) !== -1;
};
var isImmutable = function isImmutable(node) {
  return _immutable.Map.isMap(node) || _immutable.List.isList(node);
};

function splitString(path) {
  if (splitHistory[path] === undefined) {
    splitHistory[path] = path.split(DOT);
  }

  return splitHistory[path];
}

function getFromJSON(node, path) {
  var parts = splitString(path);
  var ref = node;

  for (var i = 0; i < parts.length; i += 1) {
    if (ref[parts[i]] === undefined) {
      return undefined;
    }

    ref = ref[parts[i]];
  }

  return ref;
}

function getFromImmutable(node, path) {
  return node !== undefined ? node.getIn(splitString(path)) : node;
}

function findObject(node, path, id) {
  return getFromJSON(node, path).find(function (element) {
    return element.id === id;
  });
}

function findImmutable(node, path, id) {
  return getFromImmutable(node, path).find(function (x) {
    return x.get('id') === id;
  });
}

function getArrayById(node, key, findNode, read) {
  var path = key.split(ID)[0];
  var suffix = key.split(ID).slice(1).join(ID);

  if (isInString(suffix, DOT)) {
    var _id = parseInt(suffix.split(DOT)[0], BASE);
    var subPath = suffix.replace(/^[0-9]+\./, '');

    var child = findNode(node, path, _id);
    return read(child, subPath);
  }

  var id = parseInt(suffix, BASE);
  return findNode(node, path, id);
}

var _readFromImmutable = void 0;
var _readFromJSON = void 0;
function mapObjectChildren(node, path, suffix) {
  return getFromJSON(node, path).map(function (child) {
    return _readFromJSON(child, suffix);
  });
}

function mapImmutableChildren(node, path, suffix) {
  return getFromImmutable(node, path).map(function (child) {
    return _readFromImmutable(child, suffix);
  });
}

function getChildren(node, key, mapChildren) {
  var path = key.split(AllChildren)[0];
  var suffix = key.split(AllChildren)[1];

  return mapChildren(node, path, suffix);
}

_readFromJSON = function readFromJSON(node, key) {
  if (isInString(key, ID)) {
    return getArrayById(node, key, findObject, _readFromJSON);
  } else if (isInString(key, ALL)) {
    return getChildren(node, key, mapObjectChildren);
  }

  return getFromJSON(node, key);
};

_readFromImmutable = function readFromImmutable(node, key) {
  if (isInString(key, ID)) {
    return getArrayById(node, key, findImmutable, _readFromImmutable);
  } else if (isInString(key, ALL)) {
    return getChildren(node, key, mapImmutableChildren);
  }

  return getFromImmutable(node, key);
};

function read(node, key) {
  return isImmutable(node) ? _readFromImmutable(node, key) : _readFromJSON(node, key);
}

var unwrap = exports.unwrap = function unwrap(node, key) {
  var value = read(node, key);
  return isImmutable(value) ? value.toJS() : value;
};